@@ -7,10 +7,13 @@ print_without_body <- function(x, ...) {
   }
 }
 
-if (packageVersion("pillar") >= "1.4.99") {
+if (utils::packageVersion("pillar") >= "1.4.99") {
   tbl_format_body.tbl_df_without_body <- function(x, ...) {
     "<body created by pillar>"
   }
 
-  vctrs::s3_register("pillar::tbl_format_body", "tbl_df_without_body")
+  # Need explicit method, because can't be found in method env
+  vctrs::s3_register("pillar::tbl_format_body", "tbl_df_without_body",
+    tbl_format_body.tbl_df_without_body
+  )
 }
