@@ -60,14 +60,14 @@ test_that("columns must be same length", {
 
 test_that("empty list() makes 0 x 0 tbl_df", {
   zero <- as_tibble(list())
-  expect_is(zero, "tbl_df")
+  expect_s3_class(zero, "tbl_df")
   expect_equal(dim(zero), c(0L, 0L))
 })
 
 
 test_that("NULL makes 0 x 0 tbl_df", {
   nnnull <- as_tibble(NULL)
-  expect_is(nnnull, "tbl_df")
+  expect_s3_class(nnnull, "tbl_df")
   expect_equal(dim(nnnull), c(0L, 0L))
 })
 
@@ -161,17 +161,27 @@ test_that("as_tibble() makes names `minimal`, even if not fixing names", {
 })
 
 test_that("as_tibble() implements unique names", {
-  invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "unique")
+  expect_snapshot(
+    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "unique")
+  )
   expect_equal(length(invalid_df), 3)
   expect_equal(nrow(invalid_df), 1)
-  expect_equal(names(invalid_df), vec_as_names(rep("", 3), repair = "unique"))
+  expect_equal(
+    names(invalid_df),
+    vec_as_names(rep("", 3), repair = "unique", quiet = TRUE)
+  )
 })
 
 test_that("as_tibble() implements universal names", {
-  invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "universal")
+  expect_snapshot(
+    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "universal")
+  )
   expect_equal(length(invalid_df), 3)
   expect_equal(nrow(invalid_df), 1)
-  expect_equal(names(invalid_df), vec_as_names(rep("", 3), repair = "universal"))
+  expect_equal(
+    names(invalid_df),
+    vec_as_names(rep("", 3), repair = "universal", quiet = TRUE)
+  )
 })
 
 
@@ -210,14 +220,14 @@ test_that("as_tibble.matrix() supports .name_repair", {
   x <- matrix(1:6, nrow = 3)
 
   expect_warning(as_tibble(x))
-  expect_identical(
-    names(as_tibble(x, .name_repair = "minimal")),
-    rep("", 2)
-  )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    paste0("...", 1:2)
+
+  minimal <- as_tibble(x, .name_repair = "minimal")
+  expect_identical(names(minimal), rep("", 2))
+
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), paste0("...", 1:2))
 
   x <- matrix(1:6, nrow = 3, dimnames = list(x = LETTERS[1:3], y = c("if", "when")))
 
@@ -229,10 +239,10 @@ test_that("as_tibble.matrix() supports .name_repair", {
     names(as_tibble(x, .name_repair = "minimal")),
     c("if", "when")
   )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    c(".if", "when")
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), c(".if", "when"))
 })
 
 test_that("as_tibble.poly() supports .name_repair", {
@@ -246,10 +256,10 @@ test_that("as_tibble.poly() supports .name_repair", {
     names(as_tibble(x, .name_repair = "minimal")),
     as.character(1:3)
   )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    paste0("...", 1:3)
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), paste0("...", 1:3))
 })
 
 test_that("as_tibble.table() supports .name_repair", {
@@ -263,10 +273,10 @@ test_that("as_tibble.table() supports .name_repair", {
     names(as_tibble(x, .name_repair = "minimal")),
     c("a", "a", "n")
   )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    c("a...1", "a...2", "n")
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), c("a...1", "a...2", "n"))
 
   x <- table("if" = c(1, 1, 1, 2, 2, 2), "when" = c(3, 4, 5, 3, 4, 5))
 
@@ -278,10 +288,10 @@ test_that("as_tibble.table() supports .name_repair", {
     names(as_tibble(x, .name_repair = "minimal")),
     c("if", "when", "n")
   )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    c(".if", "when", "n")
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), c(".if", "when", "n"))
 
   x <- table("m" = c(1, 1, 1, 2, 2, 2), "n" = c(3, 4, 5, 3, 4, 5))
 
@@ -289,10 +299,10 @@ test_that("as_tibble.table() supports .name_repair", {
     names(as_tibble(x, .name_repair = "minimal")),
     c("m", "n", "n")
   )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    c("m", "n...2", "n...3")
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), c("m", "n...2", "n...3"))
 })
 
 test_that("as_tibble.ts() supports .name_repair, minimal by default (#537)", {
@@ -306,10 +316,10 @@ test_that("as_tibble.ts() supports .name_repair, minimal by default (#537)", {
     names(as_tibble(x, .name_repair = "minimal")),
     rep("", 2)
   )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    paste0("...", 1:2)
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), paste0("...", 1:2))
 
   x <- ts(matrix(rnorm(6), nrow = 3), start = c(1961, 1), frequency = 12, names = c("if", "when"))
 
@@ -321,10 +331,10 @@ test_that("as_tibble.ts() supports .name_repair, minimal by default (#537)", {
     names(as_tibble(x, .name_repair = "minimal")),
     c("if", "when")
   )
-  expect_identical(
-    names(as_tibble(x, .name_repair = "universal")),
-    c(".if", "when")
+  expect_snapshot(
+    universal <- as_tibble(x, .name_repair = "universal")
   )
+  expect_identical(names(universal), c(".if", "when"))
 })
 
 test_that("as_tibble() can convert row names", {
@@ -535,27 +545,186 @@ test_that("Inconsistent `validate` and `.name_repair` used together raise a warn
   )
 })
 
-verify_output("as_tibble.txt", {
-  as_tibble(list(1))
-  as_tibble(list(1, 2))
-  as_tibble(list(a = 1, 2))
-  as_tibble(as.list(1:26))
-  as_tibble(set_names(list(1), "..1"))
-  as_tibble(set_names(list(1:26), paste0("..", 1:26)))
-  as_tibble(list(a = 1, a = 1))
-  as_tibble(list(a = 1, a = 1, b = 1, b = 1))
-  as_tibble(list(a = new_environment()))
+test_that("correct rows and cols", {
+  x <- matrix(1:6, nrow = 2)
+  out <- as_tibble(x, .name_repair = "minimal")
 
-  as_tibble_row(list(1))
-  as_tibble_row(list(1, 2))
-  as_tibble_row(list(a = 1, 2))
-  as_tibble_row(as.list(1:26))
-  as_tibble_row(set_names(list(1), "..1"))
-  as_tibble_row(set_names(list(1:26), paste0("..", 1:26)))
-  as_tibble_row(list(a = 1, a = 1))
-  as_tibble_row(list(a = 1, a = 1, b = 1, b = 1))
-  as_tibble_row(list(a = new_environment()))
-
-  as_tibble_row(list(a = 1:3))
-  as_tibble_row(list(a = 1:3, b = 1:3))
+  expect_equal(dim(out), c(2, 3))
 })
+
+test_that("correct rows and cols for 0 cols", {
+  x <- matrix(integer(), nrow = 2)
+  out <- as_tibble(x, .name_repair = "minimal")
+
+  expect_equal(dim(out), c(2, 0))
+})
+
+test_that("correct rows and cols for 0 cols and legacy naming", {
+  scoped_lifecycle_silence()
+
+  x <- matrix(integer(), nrow = 2)
+  out <- as_tibble(x)
+
+  expect_equal(dim(out), c(2, 0))
+})
+
+test_that("correct rows and cols for 0 rows", {
+  x <- matrix(integer(), ncol = 3)
+  out <- as_tibble(x, .name_repair = "minimal")
+
+  expect_equal(dim(out), c(0, 3))
+})
+
+test_that("preserves col names", {
+  x <- matrix(1:4, nrow = 2)
+  colnames(x) <- c("a", "b")
+
+  out <- as_tibble(x)
+  expect_equal(names(out), c("a", "b"))
+})
+
+test_that("supports compat col names", {
+  scoped_lifecycle_warnings()
+
+  x <- matrix(1:4, nrow = 2)
+
+  expect_warning(out <- as_tibble(x))
+  expect_equal(names(out), c("V1", "V2"))
+})
+
+test_that("creates col names with name repair", {
+  x <- matrix(1:4, nrow = 2)
+
+  expect_snapshot(
+    out <- as_tibble(x, .name_repair = "unique")
+  )
+  expect_equal(names(out), c("...1", "...2"))
+
+  expect_snapshot(
+    out <- as_tibble(x, .name_repair = "universal")
+  )
+  expect_equal(names(out), c("...1", "...2"))
+})
+
+test_that("preserves attributes except dim and names", {
+  date <- Sys.Date() + 0:3
+  dim(date) <- c(2, 2)
+  colnames(date) <- c("a", "b")
+  attr(date, "special") <- 42
+
+  out <- as_tibble.matrix(date)
+  expect_null(attributes(out[[1]])$names)
+  expect_equal(attributes(out[[1]])$class, "Date")
+  expect_equal(attributes(out[[2]])$special, 42)
+})
+
+test_that("properly handles poly class (#110)", {
+  p <- poly(1:6, 3)
+  p_df <- as_tibble(p)
+
+  expect_equal(names(p_df), colnames(p))
+  expect_equal(class(p_df[[1L]]), class(p[, 1]))
+})
+
+test_that("handles atomic vectors", {
+  x <- matrix(TRUE, nrow = 2)
+  out <- as_tibble(x, .name_repair = "minimal")
+  expect_equal(out[[1]], c(TRUE, TRUE))
+
+  x <- matrix(1L, nrow = 2)
+  out <- as_tibble(x, .name_repair = "minimal")
+  expect_equal(out[[1]], c(1L, 1L))
+
+  x <- matrix(1.5, nrow = 2)
+  out <- as_tibble(x, .name_repair = "minimal")
+  expect_equal(out[[1]], c(1.5, 1.5))
+
+  x <- matrix("a", nrow = 2)
+  out <- as_tibble(x, .name_repair = "minimal")
+  expect_equal(out[[1]], c("a", "a"))
+
+  x <- matrix(complex(real = 1, imaginary = 2), nrow = 2)
+  out <- as_tibble(x, .name_repair = "minimal")
+  expect_equal(out[[1]], as.vector(x))
+})
+
+test_that("forwarding to as.data.frame() for ts objects (#184)", {
+  mts <- cbind(
+    A = ts(c(1, 1, 2, 2),     start = 2016, frequency = 4),
+    B = ts(c(11, 11, 12, 13), start = 2016, frequency = 4)
+  )
+  expect_identical(as_tibble(mts), as_tibble(as.data.frame(mts)))
+})
+
+
+test_that("converting from matrix removes row names by default", {
+  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
+  df <- data.frame(A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
+  out <- as_tibble(x)
+  expect_false(has_rownames(out))
+  expect_identical(out, as_tibble(df))
+})
+
+test_that("converting from matrix keeps row names if argument has them, with rownames = NA", {
+  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
+  df <- data.frame(
+    A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30,
+    row.names = letters[1:6]
+  )
+
+  out <- as_tibble(x, rownames = NA)
+  expect_identical(rownames(out), rownames(x))
+  expect_identical(remove_rownames(out), as_tibble(df))
+})
+
+test_that("converting from matrix supports storing row names in a column", {
+  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
+  df <- tibble(id = letters[1:6], A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
+  out <- as_tibble(x, rownames = "id")
+  expect_identical(out, df)
+})
+
+test_that("converting from matrix uses implicit row names when `rownames =` is passed", {
+  x <- matrix(1:30, 6, 5)
+  y <- as_tibble(x, rownames = "id", .name_repair = "minimal")
+  z <- new_tibble(
+    list(
+      id = c("1", "2", "3", "4", "5", "6"),
+      c(1L, 2L, 3L, 4L, 5L, 6L),
+      c(7L, 8L, 9L, 10L, 11L, 12L),
+      c(13L, 14L, 15L, 16L, 17L, 18L),
+      c(19L, 20L, 21L, 22L, 23L, 24L),
+      c(25L, 26L, 27L, 28L, 29L, 30L)
+    ),
+    nrow = 6
+  )
+  expect_equal(y, z)
+})
+
+test_that("output test", {
+  expect_snapshot_with_error({
+    as_tibble(list(1))
+    as_tibble(list(1, 2))
+    as_tibble(list(a = 1, 2))
+    as_tibble(as.list(1:26))
+    as_tibble(set_names(list(1), "..1"))
+    as_tibble(set_names(list(1:26), paste0("..", 1:26)))
+    as_tibble(list(a = 1, a = 1))
+    as_tibble(list(a = 1, a = 1, b = 1, b = 1))
+    as_tibble(list(a = new_environment()))
+
+    as_tibble_row(list(1))
+    as_tibble_row(list(1, 2))
+    as_tibble_row(list(a = 1, 2))
+    as_tibble_row(as.list(1:26))
+    as_tibble_row(set_names(list(1), "..1"))
+    as_tibble_row(set_names(list(1:26), paste0("..", 1:26)))
+    as_tibble_row(list(a = 1, a = 1))
+    as_tibble_row(list(a = 1, a = 1, b = 1, b = 1))
+    as_tibble_row(list(a = new_environment()))
+
+    as_tibble_row(list(a = 1:3))
+    as_tibble_row(list(a = 1:3, b = 1:3))
+  })
+})
+
